#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "rolodex.h"

/*{{{  typedefs */

typedef struct ATerm _RoloList;
typedef struct ATerm _Rolodex;
typedef struct ATerm _PhoneNumber;

/*}}}  */

/*{{{  void initRolodexApi(void) */

void initRolodexApi(void)
{
  init_rolodex_dict();
}

/*}}}  */

/*{{{  term conversion functions */

/*{{{  RoloList RoloListFromTerm(ATerm t) */

RoloList RoloListFromTerm(ATerm t)
{
  return (RoloList)t;
}

/*}}}  */
/*{{{  ATerm RoloListToTerm(RoloList arg) */

ATerm RoloListToTerm(RoloList arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  Rolodex RolodexFromTerm(ATerm t) */

Rolodex RolodexFromTerm(ATerm t)
{
  return (Rolodex)t;
}

/*}}}  */
/*{{{  ATerm RolodexToTerm(Rolodex arg) */

ATerm RolodexToTerm(Rolodex arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  PhoneNumber PhoneNumberFromTerm(ATerm t) */

PhoneNumber PhoneNumberFromTerm(ATerm t)
{
  return (PhoneNumber)t;
}

/*}}}  */
/*{{{  ATerm PhoneNumberToTerm(PhoneNumber arg) */

ATerm PhoneNumberToTerm(PhoneNumber arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  constructors */

/*{{{  RoloList makeRoloListMulti(Rolodex elem, RoloList tail) */

RoloList makeRoloListMulti(Rolodex elem, RoloList tail)
{
  return (RoloList)(ATerm)ATinsert((ATermList)tail, (ATerm)elem);
}

/*}}}  */
/*{{{  RoloList makeRoloListEmpty() */

RoloList makeRoloListEmpty()
{
  return (RoloList)(ATerm)ATempty;
}

/*}}}  */
/*{{{  Rolodex makeRolodexHome(char * name, PhoneNumber phone) */

Rolodex makeRolodexHome(char * name, PhoneNumber phone)
{
  return (Rolodex)(ATerm)ATmakeAppl1(afun0, (ATerm)ATinsert(ATmakeList1((ATerm)phone), (ATerm)ATmakeAppl0(ATmakeAFun(name, 0, ATtrue))));
}

/*}}}  */
/*{{{  Rolodex makeRolodexWork(char * company, PhoneNumber phone) */

Rolodex makeRolodexWork(char * company, PhoneNumber phone)
{
  return (Rolodex)(ATerm)ATmakeAppl1(afun1, (ATerm)ATinsert(ATmakeList1((ATerm)phone), (ATerm)ATmakeAppl0(ATmakeAFun(company, 0, ATtrue))));
}

/*}}}  */
/*{{{  PhoneNumber makePhoneNumberVoice(int voice) */

PhoneNumber makePhoneNumberVoice(int voice)
{
  return (PhoneNumber)(ATerm)ATmakeAppl1(afun2, (ATerm)ATmakeInt(voice));
}

/*}}}  */
/*{{{  PhoneNumber makePhoneNumberFax(int fax) */

PhoneNumber makePhoneNumberFax(int fax)
{
  return (PhoneNumber)(ATerm)ATmakeAppl1(afun3, (ATerm)ATmakeInt(fax));
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool isEqualRoloList(RoloList arg0, RoloList arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualRolodex(Rolodex arg0, Rolodex arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool isEqualPhoneNumber(PhoneNumber arg0, PhoneNumber arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  RoloList accessors */

/*{{{  ATbool isValidRoloList(RoloList arg) */

ATbool isValidRoloList(RoloList arg)
{
  if (isRoloListMulti(arg)) {
    return ATtrue;
  }
  else if (isRoloListEmpty(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isRoloListMulti(RoloList arg) */

inline ATbool isRoloListMulti(RoloList arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternRoloListMulti, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool isRoloListEmpty(RoloList arg) */

inline ATbool isRoloListEmpty(RoloList arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, patternRoloListEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool hasRoloListElem(RoloList arg) */

ATbool hasRoloListElem(RoloList arg)
{
  if (isRoloListMulti(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  Rolodex getRoloListElem(RoloList arg) */

Rolodex getRoloListElem(RoloList arg)
{
  
    return (Rolodex)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  RoloList setRoloListElem(RoloList arg, Rolodex elem) */

RoloList setRoloListElem(RoloList arg, Rolodex elem)
{
  if (isRoloListMulti(arg)) {
    return (RoloList)ATreplace((ATermList)arg, (ATerm)elem, 0);
  }

  ATabort("RoloList has no Elem: %t\n", arg);
  return (RoloList)NULL;
}

/*}}}  */
/*{{{  ATbool hasRoloListTail(RoloList arg) */

ATbool hasRoloListTail(RoloList arg)
{
  if (isRoloListMulti(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  RoloList getRoloListTail(RoloList arg) */

RoloList getRoloListTail(RoloList arg)
{
  
    return (RoloList)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  RoloList setRoloListTail(RoloList arg, RoloList tail) */

RoloList setRoloListTail(RoloList arg, RoloList tail)
{
  if (isRoloListMulti(arg)) {
    return (RoloList)ATreplaceTail((ATermList)arg, (ATermList)tail, 1);
  }

  ATabort("RoloList has no Tail: %t\n", arg);
  return (RoloList)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  Rolodex accessors */

/*{{{  ATbool isValidRolodex(Rolodex arg) */

ATbool isValidRolodex(Rolodex arg)
{
  if (isRolodexHome(arg)) {
    return ATtrue;
  }
  else if (isRolodexWork(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isRolodexHome(Rolodex arg) */

inline ATbool isRolodexHome(Rolodex arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternRolodexHome, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isRolodexWork(Rolodex arg) */

inline ATbool isRolodexWork(Rolodex arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternRolodexWork, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasRolodexName(Rolodex arg) */

ATbool hasRolodexName(Rolodex arg)
{
  if (isRolodexHome(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char * getRolodexName(Rolodex arg) */

char * getRolodexName(Rolodex arg)
{
  
    return (char *)ATgetName(ATgetAFun((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 0))));
}

/*}}}  */
/*{{{  Rolodex setRolodexName(Rolodex arg, char * name) */

Rolodex setRolodexName(Rolodex arg, char * name)
{
  if (isRolodexHome(arg)) {
    return (Rolodex)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATmakeAppl0(ATmakeAFun(name, 0, ATtrue)), 0), 0);
  }

  ATabort("Rolodex has no Name: %t\n", arg);
  return (Rolodex)NULL;
}

/*}}}  */
/*{{{  ATbool hasRolodexPhone(Rolodex arg) */

ATbool hasRolodexPhone(Rolodex arg)
{
  if (isRolodexHome(arg)) {
    return ATtrue;
  }
  else if (isRolodexWork(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  PhoneNumber getRolodexPhone(Rolodex arg) */

PhoneNumber getRolodexPhone(Rolodex arg)
{
  if (isRolodexHome(arg)) {
    return (PhoneNumber)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 0), 1);
  }
  else 
    return (PhoneNumber)ATelementAt((ATermList)ATgetArgument((ATermAppl)arg, 0), 1);
}

/*}}}  */
/*{{{  Rolodex setRolodexPhone(Rolodex arg, PhoneNumber phone) */

Rolodex setRolodexPhone(Rolodex arg, PhoneNumber phone)
{
  if (isRolodexHome(arg)) {
    return (Rolodex)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 0), (ATerm)phone, 1), 0);
  }
  else if (isRolodexWork(arg)) {
    return (Rolodex)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 0), (ATerm)phone, 1), 0);
  }

  ATabort("Rolodex has no Phone: %t\n", arg);
  return (Rolodex)NULL;
}

/*}}}  */
/*{{{  ATbool hasRolodexCompany(Rolodex arg) */

ATbool hasRolodexCompany(Rolodex arg)
{
  if (isRolodexWork(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char * getRolodexCompany(Rolodex arg) */

char * getRolodexCompany(Rolodex arg)
{
  
    return (char *)ATgetName(ATgetAFun((ATermAppl)ATgetFirst((ATermList)ATgetArgument((ATermAppl)arg, 0))));
}

/*}}}  */
/*{{{  Rolodex setRolodexCompany(Rolodex arg, char * company) */

Rolodex setRolodexCompany(Rolodex arg, char * company)
{
  if (isRolodexWork(arg)) {
    return (Rolodex)ATsetArgument((ATermAppl)arg, (ATerm)ATreplace((ATermList)ATgetArgument((ATermAppl)arg, 0), (ATerm)ATmakeAppl0(ATmakeAFun(company, 0, ATtrue)), 0), 0);
  }

  ATabort("Rolodex has no Company: %t\n", arg);
  return (Rolodex)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  PhoneNumber accessors */

/*{{{  ATbool isValidPhoneNumber(PhoneNumber arg) */

ATbool isValidPhoneNumber(PhoneNumber arg)
{
  if (isPhoneNumberVoice(arg)) {
    return ATtrue;
  }
  else if (isPhoneNumberFax(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool isPhoneNumberVoice(PhoneNumber arg) */

inline ATbool isPhoneNumberVoice(PhoneNumber arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternPhoneNumberVoice, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool isPhoneNumberFax(PhoneNumber arg) */

inline ATbool isPhoneNumberFax(PhoneNumber arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, patternPhoneNumberFax, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool hasPhoneNumberVoice(PhoneNumber arg) */

ATbool hasPhoneNumberVoice(PhoneNumber arg)
{
  if (isPhoneNumberVoice(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  int getPhoneNumberVoice(PhoneNumber arg) */

int getPhoneNumberVoice(PhoneNumber arg)
{
  
    return (int)ATgetInt((ATermInt)ATgetArgument((ATermAppl)arg, 0));
}

/*}}}  */
/*{{{  PhoneNumber setPhoneNumberVoice(PhoneNumber arg, int voice) */

PhoneNumber setPhoneNumberVoice(PhoneNumber arg, int voice)
{
  if (isPhoneNumberVoice(arg)) {
    return (PhoneNumber)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeInt(voice), 0);
  }

  ATabort("PhoneNumber has no Voice: %t\n", arg);
  return (PhoneNumber)NULL;
}

/*}}}  */
/*{{{  ATbool hasPhoneNumberFax(PhoneNumber arg) */

ATbool hasPhoneNumberFax(PhoneNumber arg)
{
  if (isPhoneNumberFax(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  int getPhoneNumberFax(PhoneNumber arg) */

int getPhoneNumberFax(PhoneNumber arg)
{
  
    return (int)ATgetInt((ATermInt)ATgetArgument((ATermAppl)arg, 0));
}

/*}}}  */
/*{{{  PhoneNumber setPhoneNumberFax(PhoneNumber arg, int fax) */

PhoneNumber setPhoneNumberFax(PhoneNumber arg, int fax)
{
  if (isPhoneNumberFax(arg)) {
    return (PhoneNumber)ATsetArgument((ATermAppl)arg, (ATerm)ATmakeInt(fax), 0);
  }

  ATabort("PhoneNumber has no Fax: %t\n", arg);
  return (PhoneNumber)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  RoloList visitRoloList(RoloList arg, Rolodex (*acceptElem)(Rolodex)) */

RoloList visitRoloList(RoloList arg, Rolodex (*acceptElem)(Rolodex))
{
  if (isRoloListMulti(arg)) {
    return makeRoloListMulti(
        acceptElem ? acceptElem(getRoloListElem(arg)) : getRoloListElem(arg),
        visitRoloList(getRoloListTail(arg), acceptElem));
  }
  if (isRoloListEmpty(arg)) {
    return makeRoloListEmpty();
  }
  ATabort("not a RoloList: %t\n", arg);
  return (RoloList)NULL;
}

/*}}}  */
/*{{{  Rolodex visitRolodex(Rolodex arg, char * (*acceptName)(char *), PhoneNumber (*acceptPhone)(PhoneNumber), char * (*acceptCompany)(char *)) */

Rolodex visitRolodex(Rolodex arg, char * (*acceptName)(char *), PhoneNumber (*acceptPhone)(PhoneNumber), char * (*acceptCompany)(char *))
{
  if (isRolodexHome(arg)) {
    return makeRolodexHome(
        acceptName ? acceptName(getRolodexName(arg)) : getRolodexName(arg),
        acceptPhone ? acceptPhone(getRolodexPhone(arg)) : getRolodexPhone(arg));
  }
  if (isRolodexWork(arg)) {
    return makeRolodexWork(
        acceptCompany ? acceptCompany(getRolodexCompany(arg)) : getRolodexCompany(arg),
        acceptPhone ? acceptPhone(getRolodexPhone(arg)) : getRolodexPhone(arg));
  }
  ATabort("not a Rolodex: %t\n", arg);
  return (Rolodex)NULL;
}

/*}}}  */
/*{{{  PhoneNumber visitPhoneNumber(PhoneNumber arg, int (*acceptVoice)(int), int (*acceptFax)(int)) */

PhoneNumber visitPhoneNumber(PhoneNumber arg, int (*acceptVoice)(int), int (*acceptFax)(int))
{
  if (isPhoneNumberVoice(arg)) {
    return makePhoneNumberVoice(
        acceptVoice ? acceptVoice(getPhoneNumberVoice(arg)) : getPhoneNumberVoice(arg));
  }
  if (isPhoneNumberFax(arg)) {
    return makePhoneNumberFax(
        acceptFax ? acceptFax(getPhoneNumberFax(arg)) : getPhoneNumberFax(arg));
  }
  ATabort("not a PhoneNumber: %t\n", arg);
  return (PhoneNumber)NULL;
}

/*}}}  */

/*}}}  */
